import time
from pathlib import Path
from typing import List, Optional
from urllib import request

from loguru import logger

from netspresso.analytics import netspresso_analytics
from netspresso.base import NetsPressoBase
from netspresso.clients.auth import TokenHandler
from netspresso.clients.auth.response_body import UserResponse
from netspresso.clients.launcher import launcher_client_v2
from netspresso.clients.launcher.v2.schemas.task.graph_optimize.response_body import GraphOptimizeTask
from netspresso.enums import ServiceTask, Status, TaskStatusForDisplay
from netspresso.enums.graph_optimize import GraphOptimizePatternHandler
from netspresso.metadata.graph_optimizer import GraphOptimizerMetadata
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler
from netspresso.utils.onnx import update_onnx_input_batch_size_as_1


class GraphOptimizer(NetsPressoBase):
    def __init__(self, token_handler: TokenHandler, user_info: UserResponse):
        """Initialize the GraphOptimizer."""

        super().__init__(token_handler)
        self.user_info = user_info

    def _download_optimized_model(self, graph_optimize_task: GraphOptimizeTask, local_path: str) -> None:
        """Download the optimized model with given graph optimize task.

        Args:
            graph_optimize_task (GraphOptimizeTask): Graph optimize task.

        Raises:
            e: If an error occurs while getting the graph optimize task information.
        """

        self.token_handler.validate_token()

        try:
            download_url = launcher_client_v2.graph_optimizer.download_model_file(
                graph_optimize_task_id=graph_optimize_task.graph_optimize_task_id,
                access_token=self.token_handler.tokens.access_token,
            ).data.presigned_download_url

            request.urlretrieve(download_url, local_path)
            logger.info(f"Model downloaded at {Path(local_path)}")

        except Exception as e:
            logger.error(f"Download optimized model failed. Error: {e}")
            raise e

    def initialize_metadata(self, output_dir, input_model_path, pattern_handlers) -> GraphOptimizerMetadata:
        def create_metadata_with_status(status, error_message=None):
            metadata = GraphOptimizerMetadata()
            metadata.status = status
            if error_message:
                logger.error(error_message)
            return metadata

        try:
            metadata = GraphOptimizerMetadata()
        except Exception as e:
            error_message = f"An unexpected error occurred during metadata initialization: {e}"
            metadata = create_metadata_with_status(Status.ERROR, error_message)
        except KeyboardInterrupt:
            warning_message = "Graph Optimize task was interrupted by the user."
            metadata = create_metadata_with_status(Status.STOPPED, warning_message)
        finally:
            metadata.input_model_path = Path(input_model_path).resolve().as_posix()
            metadata.graph_optimize_task_info.pattern_handlers = pattern_handlers
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def optimize_model(
        self,
        input_model_path: str,
        output_dir: str,
        pattern_handlers: Optional[List[GraphOptimizePatternHandler]] = GraphOptimizePatternHandler.get_all(),
        wait_until_done: bool = True,
        sleep_interval: int = 30,
    ) -> GraphOptimizerMetadata:
        """Optimize a model to the specified framework.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local folder path to save the optimized model.
            pattern_handlers (List[GraphOptimizePatternHandler]): The pattern handlers to optimize the model.
            wait_until_done (bool): If True, wait for the graph optimize result before returning the function.
                                If False, request the graph optimize and return  the function immediately.

        Raises:
            e: If an error occurs during the graph optimize.

        Returns:
            GraphOptimizerMetadata: Graph optimize metadata.
        """

        netspresso_analytics.send_event(event_name="optimize_model_using_np")

        self.token_handler.validate_token()

        FileHandler.check_input_model_path(input_model_path)
        output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
        metadata = self.initialize_metadata(
            output_dir=output_dir,
            input_model_path=input_model_path,
            pattern_handlers=pattern_handlers,
        )

        try:
            if metadata.status in [Status.ERROR, Status.STOPPED]:
                return metadata

            # Check if the model is supported
            batch_size_1_model_path = Path(output_dir) / f"{Path(input_model_path).stem}_batch_size_1.onnx"
            input_model_path = update_onnx_input_batch_size_as_1(input_model_path, batch_size_1_model_path.as_posix())

            # Get presigned_model_upload_url
            presigned_url_response = launcher_client_v2.graph_optimizer.presigned_model_upload_url(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
            )

            # Upload model_file
            launcher_client_v2.graph_optimizer.upload_model_file(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
                presigned_upload_url=presigned_url_response.data.presigned_upload_url,
            )

            # Validate model_file
            validate_model_response = launcher_client_v2.graph_optimizer.validate_model_file(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
                ai_model_id=presigned_url_response.data.ai_model_id,
            )

            # Start graph optimize task
            graph_optimize_response = launcher_client_v2.graph_optimizer.start_task(
                access_token=self.token_handler.tokens.access_token,
                input_model_id=presigned_url_response.data.ai_model_id,
                pattern_handlers=pattern_handlers,
            )

            metadata.model_info = validate_model_response.data.to()
            metadata.graph_optimize_task_info = graph_optimize_response.data.to(validate_model_response.data.uploaded_file_name)
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

            if wait_until_done:
                while True:
                    self.token_handler.validate_token()
                    graph_optimize_response = launcher_client_v2.graph_optimizer.read_task(
                        access_token=self.token_handler.tokens.access_token,
                        task_id=graph_optimize_response.data.graph_optimize_task_id,
                    )
                    if graph_optimize_response.data.status in [
                        TaskStatusForDisplay.FINISHED,
                        TaskStatusForDisplay.ERROR,
                        TaskStatusForDisplay.TIMEOUT,
                    ]:
                        break

                    time.sleep(sleep_interval)

            if graph_optimize_response.data.status == TaskStatusForDisplay.FINISHED:
                default_model_path = FileHandler.get_default_model_path(folder_path=output_dir)
                extension = ".onnx"
                self._download_optimized_model(
                    graph_optimize_task=graph_optimize_response.data,
                    local_path=str(default_model_path.with_suffix(extension)),
                )
                metadata.status = Status.COMPLETED
                metadata.optimized_model_path = default_model_path.with_suffix(extension).as_posix()
                metadata.graph_optimize_task_info = graph_optimize_response.data.to(validate_model_response.data.uploaded_file_name)
                logger.info("Graph Optimize task was completed successfully.")
            else:
                metadata = self.handle_error(metadata, ServiceTask.MODEL_GRAPH_OPTIMIZE, graph_optimize_response.data.error_log)

        except Exception as e:
            metadata = self.handle_error(metadata, ServiceTask.MODEL_GRAPH_OPTIMIZE, e.args[0])
        except KeyboardInterrupt:
            metadata = self.handle_stop(metadata, ServiceTask.MODEL_GRAPH_OPTIMIZE)
        finally:
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def get_graph_optimize_task(self, graph_optimize_task_id: str) -> GraphOptimizeTask:
        """Get the graph optimize task information with given graph optimize task uuid.

        Args:
            graph_optimize_task_id (str): Graph optimize task UUID of the graph optimize task.

        Raises:
            e: If an error occurs during the graph optimize task.

        Returns:
            GraphOptimizeTask: Graph optimize task dictionary.
        """

        self.token_handler.validate_token()

        response = launcher_client_v2.graph_optimizer.read_task(
            access_token=self.token_handler.tokens.access_token,
            task_id=graph_optimize_task_id,
        )
        return response.data



from .request_body import RequestCreateSimulateTask
from .response_body import (
    ResponseSimulateStatusItem,
    ResponseSimulateTaskItem,
)

__all__ = [
    "RequestCreateSimulateTask",
    "ResponseSimulateTaskItem",
    "ResponseSimulateStatusItem",
]



from aenum import NamedConstant

from netspresso.enums.base import StrEnum


class Task(StrEnum):
    IMAGE_CLASSIFICATION = "classification"
    OBJECT_DETECTION = "detection"
    SEMANTIC_SEGMENTATION = "segmentation"


class LauncherTask(StrEnum):
    CONVERT = "convert"
    BENCHMARK = "benchmark"
    QUANTIZE = "quantize"
    GRAPH_OPTIMIZE = "graph-optimize"
    SIMULATE = "simulate"


class TaskStatusForDisplay(NamedConstant):
    # task status for display - launchx
    IN_QUEUE = "IN_QUEUE"
    IN_PROGRESS = "IN_PROGRESS"
    FINISHED = "FINISHED"
    ERROR = "ERROR"
    USER_CANCEL = "USER_CANCEL"
    TIMEOUT = "TIMEOUT"



import os

from loguru import logger

from netspresso.clients.launcher.v2.implements import ModelAPI, SimulateTaskAPI
from netspresso.clients.launcher.v2.schemas import (
    AuthorizationHeader,
    RequestModelUploadUrl,
    RequestUploadModel,
    RequestValidateModel,
    ResponseModelItem,
    ResponseModelOptions,
    ResponseModelUploadUrl,
    UploadFile,
)
from netspresso.clients.launcher.v2.schemas.common import UploadDataset
from netspresso.clients.launcher.v2.schemas.task.simulate.request_body import RequestCreateSimulateTask
from netspresso.clients.launcher.v2.schemas.task.simulate.response_body import (
    ResponseSimulateStatusItem,
    ResponseSimulateTaskItem,
)
from netspresso.clients.utils.common import read_file_bytes
from netspresso.enums import LauncherTask
from netspresso.enums.simulate import SimulateTaskType


class Simulator:
    def __init__(self, url):
        self.task_type = LauncherTask.SIMULATE.value
        self.simulate_task = SimulateTaskAPI(url=url)
        self.simulate_model = ModelAPI(url=url, task_type=self.task_type)

    def presigned_model_upload_url(self, access_token: str, input_model_path: str) -> ResponseModelUploadUrl:
        object_name = os.path.basename(input_model_path)

        get_upload_url_request_body = RequestModelUploadUrl(object_name=object_name, task=self.task_type)
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Simulate model upload: path - {input_model_path}")

        upload_url_response_body = self.simulate_model.get_upload_url(
            request_params=get_upload_url_request_body, headers=token_header
        )
        logger.info(f"Request Simulate upload_url result: {upload_url_response_body}")
        return upload_url_response_body

    def upload_model_file(self, access_token: str, input_model_path: str, presigned_upload_url: str) -> str:
        object_name = os.path.basename(input_model_path)
        file_content = read_file_bytes(file_path=input_model_path)

        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(
            f"Request Simulate model validation:"
            f" path | {input_model_path} "
            f" presigned_upload_url | {presigned_upload_url}"
        )

        get_model_upload_request_body = RequestUploadModel(url=presigned_upload_url)
        model_file_object = UploadFile(file_name=object_name, file_content=file_content)

        upload_result = self.simulate_model.upload(
            request_body=get_model_upload_request_body,
            file=model_file_object,
            headers=token_header,
        )
        logger.info(f"Request Simulate upload_model_file result: {upload_result}")
        return upload_result

    def validate_model_file(self, access_token: str, input_model_path: str, ai_model_id: str) -> ResponseModelItem:
        object_name = os.path.basename(input_model_path)
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Simulate model validation:" f" path - {input_model_path}" f" ai_model_id - {ai_model_id}")

        get_validate_model_request_body = RequestValidateModel(
            ai_model_id=ai_model_id,
            display_name=object_name,
        )

        validated_model = self.simulate_model.validate(
            request_body=get_validate_model_request_body, headers=token_header
        )
        logger.info(f"Request Simulate validate_model result: {validated_model}")
        return validated_model

    def read_model_task_options(self, access_token, ai_model_id) -> ResponseModelOptions:
        token_header = AuthorizationHeader(access_token=access_token)
        model_task_options = self.simulate_model.options(headers=token_header, ai_model_id=ai_model_id)
        logger.info(f"Request model task_options: {model_task_options}")
        return model_task_options

    def start_task(
        self,
        access_token: str,
        base_model_id: str,
        target_model_id: str,
        type: SimulateTaskType,
        dataset_path: str = None,
    ) -> ResponseSimulateTaskItem:
        token_header = AuthorizationHeader(access_token=access_token)
        request_body = RequestCreateSimulateTask(
            base_model_id=base_model_id,
            target_model_id=target_model_id,
            type=type,
        )
        logger.info(f"Request Simulate body: {request_body}")

        if dataset_path:
            dataset_filename = os.path.basename(dataset_path)
            file_object = UploadDataset(
                file_name=dataset_filename,
                file_content=read_file_bytes(dataset_path),
            )
        else:
            file_object = None

        simulate_task_response = self.simulate_task.start(
            request_body=request_body, headers=token_header, file=file_object
        )
        logger.info(f"Request Simulate result: {simulate_task_response}")
        return simulate_task_response

    def cancel_task(self, access_token: str, task_id: str) -> ResponseSimulateTaskItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Simulate Cancel task_id: {task_id}")

        simulate_task_response = self.simulate_task.cancel(headers=token_header, task_id=task_id)
        logger.info(f"Request Simulate Cancel result: {simulate_task_response}")
        return simulate_task_response

    def read_task(self, access_token: str, task_id: str) -> ResponseSimulateTaskItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Simulate Read task_id: {task_id}")

        simulate_task_response = self.simulate_task.read(headers=token_header, task_id=task_id)
        logger.info(f"Request Simulate Task Info: {simulate_task_response}")
        return simulate_task_response

    def delete_task(self, access_token: str, task_id: str) -> ResponseSimulateTaskItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Simulate delete task_id: {task_id}")

        simulate_task_response = self.simulate_task.delete(headers=token_header, task_id=task_id)
        logger.info(f"Request Simulate Delete Info: {simulate_task_response}")
        return simulate_task_response

    def read_status(self, access_token: str, task_id: str) -> ResponseSimulateStatusItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Simulate read_status task_id: {task_id}")

        simulate_task_response = self.simulate_task.status(headers=token_header, task_id=task_id)
        logger.info(f"Request Simulate Task Status: {simulate_task_response}")
        return simulate_task_response



from dataclasses import dataclass, field
from typing import List

from netspresso.clients.launcher.v2.schemas import InputLayer
from netspresso.enums.graph_optimize import GraphOptimizePatternHandler
from netspresso.enums.metadata import TaskType
from netspresso.enums.task import TaskStatusForDisplay
from netspresso.metadata.common import BaseMetadata, ModelInfo


@dataclass
class GraphOptimizeInfo:
    graph_optimize_task_id: str = ""
    input_model_id: str = ""
    user_id: str = ""
    output_model_id: str = ""
    model_file_name: str = ""
    pattern_handlers: List[GraphOptimizePatternHandler] = field(default_factory=list)
    status: TaskStatusForDisplay = ""
    error_log: str = ""
    input_layer: List[InputLayer] = field(default_factory=list)


@dataclass
class GraphOptimizerMetadata(BaseMetadata):
    task_type: TaskType = TaskType.GRAPH_OPTIMIZE
    input_model_path: str = ""
    optimized_model_path: str = ""
    model_info: ModelInfo = field(default_factory=ModelInfo)
    graph_optimize_task_info: GraphOptimizeInfo = field(default_factory=GraphOptimizeInfo)



from netspresso.clients.config import Config, ServiceModule, ServiceName
from netspresso.clients.launcher.v2.benchmarker import Benchmarker
from netspresso.clients.launcher.v2.converter import Converter
from netspresso.clients.launcher.v2.graph_optimizer import GraphOptimizer
from netspresso.clients.launcher.v2.quantizer import Quantizer
from netspresso.clients.launcher.v2.simulator import Simulator


class LauncherAPIClient:
    def __init__(self):
        self.config = Config(ServiceName.NP, ServiceModule.LAUNCHER)
        self.host = self.config.HOST
        self.port = self.config.PORT
        self.prefix = self.config.URI_PREFIX
        self.url = f"{self.host}:{self.port}{self.prefix}"
        self.converter = self._create_convert_api()
        self.benchmarker = self._create_benchmark_api()
        self.quantizer = self._create_quantize_api()
        self.graph_optimizer = self._create_graph_optimizer_api()
        self.simulator = self._create_simulator_api()

    def _create_convert_api(self):
        return Converter(self.url)

    def _create_benchmark_api(self):
        return Benchmarker(self.url)

    def _create_quantize_api(self):
        return Quantizer(self.url)

    def _create_graph_optimizer_api(self):
        return GraphOptimizer(self.url)

    def _create_simulator_api(self):
        return Simulator(self.url)

    def is_cloud(self) -> bool:
        return self.config.is_cloud()


launcher_client_v2 = LauncherAPIClient()



from .request_body import RequestCreateGraphOptimizeTask
from .response_body import (
    ResponseGraphOptimizeDownloadModelUrlItem,
    ResponseGraphOptimizeStatusItem,
    ResponseGraphOptimizeTaskItem,
)

__all__ = [
    "RequestCreateGraphOptimizeTask",
    "ResponseGraphOptimizeTaskItem",
    "ResponseGraphOptimizeStatusItem",
    "ResponseGraphOptimizeDownloadModelUrlItem",
]



from netspresso.enums.base import StrEnum


class TaskType(StrEnum):
    TRAIN = "train"
    COMPRESS = "compress"
    CONVERT = "convert"
    QUANTIZE = "quantize"
    BENCHMARK = "benchmark"
    GRAPH_OPTIMIZE = "graph_optimize"
    PROFILE = "profile"
    SIMULATE = "simulate"


class Status(StrEnum):
    IN_PROGRESS = "in_progress"
    COMPLETED = "completed"
    STOPPED = "stopped"
    ERROR = "error"



from dataclasses import dataclass, field
from typing import List

from netspresso.enums.graph_optimize import GraphOptimizePatternHandler


@dataclass
class RequestCreateGraphOptimizeTask:
    ai_model_id: str
    pattern_handlers: List[GraphOptimizePatternHandler] = field(
        default_factory=GraphOptimizePatternHandler.get_all
    )



from dataclasses import dataclass, field
from typing import List, Optional

from loguru import logger

from netspresso.enums.compression import GroupPolicy, LayerNorm, Policy, StepOp
from netspresso.enums.device import DeviceName
from netspresso.exceptions.compressor import NotValidChannelAxisRangeException


@dataclass
class OptionsBase:
    reshape_channel_axis: int = -1

    def __post_init__(self):
        valid_values = [0, 1, -1, -2]
        if self.reshape_channel_axis not in valid_values:
            raise NotValidChannelAxisRangeException(self.reshape_channel_axis)


@dataclass
class Options(OptionsBase):
    policy: Policy = Policy.AVERAGE
    layer_norm: LayerNorm = LayerNorm.STANDARD_SCORE
    group_policy: GroupPolicy = GroupPolicy.AVERAGE
    step_size: int = 2
    step_op: StepOp = StepOp.ROUND
    reverse: bool = False
    target_device_name: Optional[str] = None

    def __post_init__(self):
        super().__post_init__()

        # Apply hardware-specific step size configuration
        if self.target_device_name is not None:
            self._apply_hardware_config()

    def _apply_hardware_config(self):
        """Apply hardware-specific step size and operation configuration for target device."""
        if self.target_device_name is None:
            return

        # Normalize device name (case-insensitive matching)
        device_name_lower = self.target_device_name.lower()

        # Check if target device is NXP iMX93
        if device_name_lower in [DeviceName.NXP_iMX93.lower(), DeviceName.NXP_iMX93.value.lower()]:
            self.step_size = 8
            self.step_op = StepOp.ROUND_DOWN
            logger.info(f"🎯 Hardware-aware compression enabled for {DeviceName.NXP_iMX93.value}")
            logger.info(f"   → step_size: {self.step_size}, step_op: {self.step_op.value}")
            logger.info("   → Reason: NPU requires 8-channel alignment for optimal performance")


@dataclass
class RecommendationOptions(Options):
    min_num_of_value: int = 8


@dataclass
class Layer:
    use: bool = False
    name: str = "input"
    channels: List[int] = field(default_factory=list)
    values: List[int] = field(default_factory=list)



from .model import ModelAPI
from .task import BenchmarkTaskAPI, ConvertTaskAPI, GraphOptimizeTaskAPI, QuantizeTaskAPI, SimulateTaskAPI

__all__ = [ModelAPI, BenchmarkTaskAPI, ConvertTaskAPI, GraphOptimizeTaskAPI, QuantizeTaskAPI, SimulateTaskAPI]



from .benchmark_task import BenchmarkTaskAPI
from .convert_task import ConvertTaskAPI
from .graph_optimize_task import GraphOptimizeTaskAPI
from .quantize_task import QuantizeTaskAPI
from .simulate_task import SimulateTaskAPI

__all__ = [BenchmarkTaskAPI, ConvertTaskAPI, QuantizeTaskAPI, GraphOptimizeTaskAPI, SimulateTaskAPI]



from pathlib import Path
from typing import Optional

from loguru import logger

from netspresso.analytics import netspresso_analytics
from netspresso.base import NetsPressoBase
from netspresso.clients.auth import TokenHandler
from netspresso.clients.auth.response_body import UserResponse
from netspresso.clients.launcher import launcher_client_v2
from netspresso.clients.launcher.v2.schemas.model.response_body import ResponseModelItem
from netspresso.enums import ServiceTask, Status
from netspresso.enums.simulate import SimulateTaskType
from netspresso.metadata.simulator import SimulatorMetadata
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class Simulator(NetsPressoBase):
    def __init__(self, token_handler: TokenHandler, user_info: UserResponse):
        """Initialize the Simulator."""

        super().__init__(token_handler)
        self.user_info = user_info

    def initialize_metadata(self, output_dir, base_model_path, target_model_path) -> SimulatorMetadata:
        def create_metadata_with_status(status, error_message=None):
            metadata = SimulatorMetadata()
            metadata.status = status
            if error_message:
                logger.error(error_message)
            return metadata

        try:
            metadata = SimulatorMetadata()
        except Exception as e:
            error_message = f"An unexpected error occurred during metadata initialization: {e}"
            metadata = create_metadata_with_status(Status.ERROR, error_message)
        except KeyboardInterrupt:
            warning_message = "Simulator task was interrupted by the user."
            metadata = create_metadata_with_status(Status.STOPPED, warning_message)
        finally:
            metadata.base_model_path = Path(base_model_path).resolve().as_posix()
            metadata.target_model_path = Path(target_model_path).resolve().as_posix()
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def _upload_model_file(self, model_path: str) -> ResponseModelItem:
        # Get presigned_model_upload_url
        presigned_url_response = launcher_client_v2.simulator.presigned_model_upload_url(
            access_token=self.token_handler.tokens.access_token,
            input_model_path=model_path,
        )

        # Upload model_file
        launcher_client_v2.simulator.upload_model_file(
            access_token=self.token_handler.tokens.access_token,
            input_model_path=model_path,
            presigned_upload_url=presigned_url_response.data.presigned_upload_url,
        )

        # Validate model_file
        validate_model_response = launcher_client_v2.simulator.validate_model_file(
            access_token=self.token_handler.tokens.access_token,
            input_model_path=model_path,
            ai_model_id=presigned_url_response.data.ai_model_id,
        )

        return validate_model_response

    def simulate_model(
        self,
        base_model_path: str,
        target_model_path: str,
        output_dir: str,
        dataset_path: Optional[str] = None,
    ) -> SimulatorMetadata:
        """Simulate a model to the specified framework.

        Args:
            base_model_path (str): The file path where the base model is located.
            target_model_path (str): The file path where the target model is located.
            output_dir (str): The local folder path to save the simulation result.
            dataset_path (str): The file path where the dataset is located.

        Raises:
            e: If an error occurs during the simulation.

        Returns:
            SimulatorMetadata: Simulator metadata.
        """

        netspresso_analytics.send_event(event_name="simulate_model_using_np")

        FileHandler.check_input_model_path(base_model_path)
        FileHandler.check_input_model_path(target_model_path)
        output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
        metadata = self.initialize_metadata(
            output_dir=output_dir,
            base_model_path=base_model_path,
            target_model_path=target_model_path,
        )

        try:
            if metadata.status in [Status.ERROR, Status.STOPPED]:
                return metadata

            base_model = self._upload_model_file(base_model_path)
            target_model = self._upload_model_file(target_model_path)

            # Start simulate task
            simulate_response = launcher_client_v2.simulator.start_task(
                access_token=self.token_handler.tokens.access_token,
                base_model_id=base_model.data.ai_model_id,
                target_model_id=target_model.data.ai_model_id,
                type=SimulateTaskType.OUTPUT,
                dataset_path=dataset_path,
            )

            metadata.base_model_info = base_model.data.to()
            metadata.target_model_info = target_model.data.to()
            metadata.simulate_task_info = simulate_response.data.to()
            metadata.status = Status.COMPLETED

        except Exception as e:
            metadata = self.handle_error(metadata, ServiceTask.MODEL_SIMULATE, e.args[0])
        except KeyboardInterrupt:
            metadata = self.handle_stop(metadata, ServiceTask.MODEL_SIMULATE)
        finally:
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata



from dataclasses import asdict
from enum import Enum
from typing import Optional

from loguru import logger

from netspresso.clients.launcher.v2.interfaces import TaskInterface
from netspresso.clients.launcher.v2.schemas import (
    AuthorizationHeader,
    RequestCreateGraphOptimizeTask,
    ResponseGraphOptimizeDownloadModelUrlItem,
    ResponseGraphOptimizeStatusItem,
    ResponseGraphOptimizeTaskItem,
    UploadFile,
)
from netspresso.clients.utils.requester import Requester
from netspresso.enums import LauncherTask


class GraphOptimizeTaskAPI(TaskInterface):
    def __init__(self, url):
        self.task_type = LauncherTask.GRAPH_OPTIMIZE.value
        self.base_url = url
        self.task_base_url = f"{self.base_url}/{self.task_type}/tasks"
        self.model_base_url = f"{self.base_url}/{self.task_type}/models"

    @staticmethod
    def custom_asdict_factory(data):
        def convert_value(obj):
            if isinstance(obj, Enum):
                return obj.value
            return obj

        return {k: convert_value(v) for k, v in data}

    def start(
        self,
        request_body: RequestCreateGraphOptimizeTask,
        headers: AuthorizationHeader,
        file: Optional[UploadFile] = None,
    ) -> ResponseGraphOptimizeTaskItem:
        endpoint = f"{self.task_base_url}"

        logger.info(f"Request_Body: {asdict(request_body)}")
        response = Requester().post_as_json(url=endpoint, request_body=asdict(request_body), headers=headers.to_dict())
        return ResponseGraphOptimizeTaskItem(**response.json())

    def cancel(self, headers: AuthorizationHeader, task_id: str) -> ResponseGraphOptimizeTaskItem:
        endpoint = f"{self.task_base_url}/{task_id}/cancel"
        response = Requester().post_as_json(url=endpoint, request_body={}, headers=headers.to_dict())
        return ResponseGraphOptimizeTaskItem(**response.json())

    def read(self, headers: AuthorizationHeader, task_id: str) -> ResponseGraphOptimizeTaskItem:
        endpoint = f"{self.task_base_url}/{task_id}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseGraphOptimizeTaskItem(**response.json())

    def status(self, headers: AuthorizationHeader, task_id: str) -> ResponseGraphOptimizeStatusItem:
        endpoint = f"{self.task_base_url}/{task_id}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseGraphOptimizeStatusItem(**response.json())

    def get_download_url(
        self, headers: AuthorizationHeader, graph_optimize_task_uuid: str
    ) -> ResponseGraphOptimizeDownloadModelUrlItem:
        endpoint = f"{self.task_base_url}/{graph_optimize_task_uuid}/models/download"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseGraphOptimizeDownloadModelUrlItem(**response.json())

    def options(self, headers: AuthorizationHeader):
        pass

    def options_by_model_framework(self, headers: AuthorizationHeader, model_framework: str):
        pass

    def delete(self, headers: AuthorizationHeader, task_id: str):
        pass



import dataclasses
from dataclasses import dataclass, field
from datetime import datetime
from typing import List, Optional

from netspresso.clients.launcher.v2.schemas import InputLayer, ResponseItem
from netspresso.clients.launcher.v2.schemas.task.common import TaskStatusInfo
from netspresso.enums import TaskStatusForDisplay
from netspresso.enums.graph_optimize import GraphOptimizePatternHandler
from netspresso.metadata.graph_optimizer import GraphOptimizeInfo


@dataclass
class GraphOptimizeTask:
    graph_optimize_task_id: str
    input_model_id: str
    user_id: str
    output_model_id: str
    model_file_name: str
    pattern_handlers: List[GraphOptimizePatternHandler]
    status: TaskStatusForDisplay = ""
    error_log: Optional[dict] = None
    created_at: datetime = field(default_factory=datetime.now)
    input_layer: InputLayer = field(default_factory=InputLayer)

    def __init__(self, **kwargs):
        names = {f.name for f in dataclasses.fields(self)}
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)

        self.input_layer = InputLayer(**self.input_layer)

    def to(self, model_file_name: str) -> GraphOptimizeInfo:
        graph_optimize_info = GraphOptimizeInfo()
        graph_optimize_info.graph_optimize_task_id = self.graph_optimize_task_id
        graph_optimize_info.input_model_id = self.input_model_id
        graph_optimize_info.user_id = self.user_id
        graph_optimize_info.output_model_id = self.output_model_id
        graph_optimize_info.model_file_name = model_file_name
        graph_optimize_info.pattern_handlers = self.pattern_handlers
        graph_optimize_info.status = self.status
        graph_optimize_info.error_log = self.error_log
        graph_optimize_info.input_layer = self.input_layer

        return graph_optimize_info


@dataclass
class ResponseGraphOptimizeTaskItem(ResponseItem):
    data: Optional[GraphOptimizeTask] = field(default_factory=dict)

    def __post_init__(self):
        self.data = GraphOptimizeTask(**self.data)


@dataclass
class ResponseGraphOptimizeStatusItem(ResponseItem):
    data: TaskStatusInfo = field(default_factory=TaskStatusInfo)

    def __post_init__(self):
        self.data = TaskStatusInfo(**self.data)


@dataclass
class DownloadModelUrl:
    ai_model_id: str
    presigned_download_url: str


@dataclass
class ResponseGraphOptimizeDownloadModelUrlItem(ResponseItem):
    data: Optional[DownloadModelUrl] = field(default_factory=dict)

    def __post_init__(self):
        self.data = DownloadModelUrl(**self.data)



from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional

from netspresso.clients.auth.v2.schemas.common import ApiKeyStatus


@dataclass
class TokenRefreshRequest:
    refresh_token: str


@dataclass
class ApiKeyPayload:
    access_key: str
    status: ApiKeyStatus
    deactivated_at: Optional[datetime] = None
    created_time: Optional[datetime] = None


@dataclass
class ApiKeyResponse:
    data: Optional[ApiKeyPayload] = field(default_factory=ApiKeyPayload)

    def __post_init__(self):
        self.data = ApiKeyPayload(**self.data)



import dataclasses
from dataclasses import dataclass, field
from datetime import datetime
from typing import List, Optional, Union

from netspresso.clients.launcher.v2.schemas import ResponseItem
from netspresso.clients.launcher.v2.schemas.task.common import TaskStatusInfo
from netspresso.enums import TaskStatusForDisplay
from netspresso.enums.simulate import SimulateTaskType
from netspresso.metadata.simulator import SimulatorInfo


@dataclass
class SnrScore:
    layer_name: str
    snr_score: Union[float, str]
    sensitivity_score: float
    sensitivity_rank: Optional[int]


@dataclass
class SimulateTask:
    simulate_task_id: str
    user_id: str
    base_model_id: str
    target_model_id: str
    snr_scores: List[SnrScore]
    type: SimulateTaskType
    status: TaskStatusForDisplay = ""
    error_log: Optional[dict] = None
    created_at: datetime = field(default_factory=datetime.now)

    def __init__(self, **kwargs):
        names = {f.name for f in dataclasses.fields(self)}
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)

    def to(self) -> SimulatorInfo:
        simulator_info = SimulatorInfo()
        simulator_info.simulate_task_id = self.simulate_task_id
        simulator_info.user_id = self.user_id
        simulator_info.base_model_id = self.base_model_id
        simulator_info.target_model_id = self.target_model_id
        simulator_info.snr_scores = self.snr_scores
        simulator_info.type = self.type
        simulator_info.status = self.status
        simulator_info.error_log = self.error_log

        return simulator_info


@dataclass
class ResponseSimulateTaskItem(ResponseItem):
    data: Optional[SimulateTask] = field(default_factory=dict)

    def __post_init__(self):
        self.data = SimulateTask(**self.data)


@dataclass
class ResponseSimulateStatusItem(ResponseItem):
    data: TaskStatusInfo = field(default_factory=TaskStatusInfo)

    def __post_init__(self):
        self.data = TaskStatusInfo(**self.data)



from dataclasses import dataclass, field
from typing import List

from netspresso.enums.metadata import TaskType
from netspresso.enums.simulate import SimulateTaskType
from netspresso.enums.task import TaskStatusForDisplay
from netspresso.metadata.common import BaseMetadata, ModelInfo


@dataclass
class SimulatorInfo:
    simulate_task_id: str = ""
    user_id: str = ""
    base_model_id: str = ""
    target_model_id: str = ""
    snr_scores: List[dict] = field(default_factory=list)
    type: SimulateTaskType = ""
    status: TaskStatusForDisplay = ""
    error_log: str = ""


@dataclass
class SimulatorMetadata(BaseMetadata):
    task_type: TaskType = TaskType.SIMULATE
    base_model_path: str = ""
    target_model_path: str = ""
    base_model_info: ModelInfo = field(default_factory=ModelInfo)
    target_model_info: ModelInfo = field(default_factory=ModelInfo)
    simulate_task_info: SimulatorInfo = field(default_factory=SimulatorInfo)



import json

import onnx
from google.protobuf.json_format import MessageToJson, Parse

from netspresso.exceptions.graph_optimizer import LoadONNXModelException, UpdateOnnxException


def update_dim_value(onnx_json, idx_list, key):
    for idx in idx_list:
        tensor_dim = onnx_json["graph"][key][idx]["type"]["tensorType"]["shape"]["dim"][0]
        tensor_dim["dimValue"] = 1
        if tensor_dim.get("dimParam"):
            del tensor_dim["dimParam"]
    return onnx_json


def find_initializer_idx_list_onnx(onnx_json):
    graph = onnx_json["graph"]
    not_filtered_inputs = graph["input"]  # it contains initialized variables

    initializers = graph["initializer"]
    initializer_names = [x["name"] for x in initializers]

    filtered_input_idx_list = []
    for i in range(len(not_filtered_inputs)):
        if graph["input"][i]["name"] not in initializer_names:
            filtered_input_idx_list.append(i)

    return filtered_input_idx_list


def load_onnx_model(model_path):
    try:
        onnx_model = onnx.load(model_path)
    except Exception as e:
        raise LoadONNXModelException(error_log=e.__repr__()) from e

    return onnx_model


def update_onnx_input_batch_size_as_1(model_path, save_path):
    onnx_model = load_onnx_model(model_path=model_path)

    try:
        s = MessageToJson(onnx_model)

        # set batch size as 1
        onnx_json = json.loads(s)

        # find input
        input_idx_list = find_initializer_idx_list_onnx(onnx_json)
        onnx_json = update_dim_value(onnx_json, input_idx_list, "input")
        output_idx_list = list(range(len(onnx_json["graph"]["output"])))
        onnx_json = update_dim_value(onnx_json, output_idx_list, "output")

        onnx_str = json.dumps(onnx_json)
        converted_model = Parse(onnx_str, onnx.ModelProto())
        onnx.save(converted_model, save_path)

        return save_path

    except Exception as e:
        raise UpdateOnnxException(error_log=e.__repr__()) from e



import os
import shutil
import zipfile
from pathlib import Path
from typing import List, Optional, Union

import onnx
import qai_hub as hub
from loguru import logger
from qai_hub.client import (
    CompileJob,
    Dataset,
    Device,
    Job,
    JobStatus,
    JobSummary,
    JobType,
    Model,
    QuantizeJob,
    SourceModel,
    SourceModelType,
)

from netspresso.np_qai.options import Extension, Framework, Runtime


class NPQAIBase:
    def set_verbose(self, verbose: bool) -> None:
        hub.set_verbose(verbose)

    def upload_dataset(self, data, name=None) -> Dataset:
        """
        Upload a dataset to the QAI Hub.

        Args:
            data: The dataset to upload.
            name: The name of the dataset.

        Returns:
            Dataset: Returns a dataset object if successful.

        Note:
            For details, see [upload_dataset in QAI Hub API](https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.upload_dataset.html).
        """
        dataset = hub.upload_dataset(data=data, name=name)

        return dataset

    def get_dataset(self, dataset_id: str) -> Dataset:
        """
        Get a dataset from the QAI Hub.

        Args:
            dataset_id: The ID of the dataset to get.

        Returns:
            Dataset: Returns a dataset object if successful.

        Note:
            For details, see [get_dataset in QAI Hub API](https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_dataset.html).
        """
        dataset = hub.get_dataset(dataset_id=dataset_id)

        return dataset

    def get_datasets(self, offset: int = 0, limit: int = 50) -> List[Dataset]:
        """
        Get a list of datasets from the QAI Hub.

        Args:
            offset: The offset of the datasets to get even older datasets.
            limit: The limit of the datasets to get.

        Returns:
            List[Dataset]: Returns a list of dataset objects if successful.

        Note:
            For details, see [get_datasets in QAI Hub API](https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_datasets.html).
        """
        datasets = hub.get_datasets(offset=offset, limit=limit)

        return datasets

    def upload_model(self, model: Union[SourceModel, str], name: Optional[str] = None) -> Model:
        """
        Upload a model to the QAI Hub.

        Args:
            model: The model to upload.
            name: The name of the model.

        Returns:
            Model: Returns a model object if successful.

        Note:
            For details, see [upload_model in QAI Hub API](https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.upload_model.html).
        """
        model = hub.upload_model(model=model, name=name)

        return model

    def get_models(self, offset: int = 0, limit: int = 50) -> List[Model]:
        """
        Get a list of models from the QAI Hub.

        Args:
            offset: The offset of the models to get even older models.
            limit: The limit of the models to get.

        Returns:
            List[Model]: Returns a list of model objects if successful.

        Note:
            For details, see [get_models in QAI Hub API](https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_models.html).
        """
        models = hub.get_models(offset=offset, limit=limit)

        return models

    def get_model(self, model_id: str) -> Model:
        """
        Get a model from the QAI Hub.

        Args:
            model_id: The ID of the model to get.

        Returns:
            Model: Returns a model object if successful.

        Note:
            For details, see [get_model in QAI Hub API](https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_model.html).
        """
        model = hub.get_model(model_id=model_id)

        return model

    def get_devices(self, name: str = "", os: str = "", attributes: Union[str, List[str]] = None) -> List[Device]:
        """
        Get a list of devices from the QAI Hub.

        Args:
            name: The name of the device to get.
            os: The OS of the device to get.
            attributes: The attributes of the device to get.

        Returns:
            List[Device]: Returns a list of device objects if successful.

        Note:
            For details, see [get_devices in QAI Hub API](https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_devices.html).
        """
        if attributes is None:
            attributes = []
        devices = hub.get_devices(name=name, os=os, attributes=attributes)

        return devices

    def get_device_attributes(self) -> List[str]:
        """
        Get a list of device attributes from the QAI Hub.

        Returns:
            List[str]: Returns a list of device attribute strings if successful.

        Note:
            For details, see [get_device_attributes in QAI Hub API](https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_device_attributes.html).
        """
        device_attributes = hub.get_device_attributes()

        return device_attributes

    def get_job_summaries(
        self,
        offset: int = 0,
        limit: int = 50,
        creator: Optional[str] = None,
        state: Union[Optional[JobStatus.State], List[JobStatus.State]] = None,
        type: Optional[JobType] = None,
    ) -> List[JobSummary]:
        """
        Get a list of job summaries from the QAI Hub.

        Returns:
            List[JobSummary]: Returns a list of job summary objects if successful.

        Note:
            For details, see [get_job_summaries in QAI Hub API](https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_job_summaries.html).
        """
        job_summaries = hub.get_job_summaries(offset=offset, limit=limit, creator=creator, state=state, type=type)

        return job_summaries

    def get_job(self, job_id: str) -> Job:
        """
        Get a job from the QAI Hub.

        Args:
            job_id: The ID of the job to get.

        Returns:
            Job: Returns a job object if successful.

        Note:
            For details, see [get_job in QAI Hub API](https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_job.html).
        """
        job = hub.get_job(job_id=job_id)

        return job

    def get_source_extension(self, model_path):
        extension = Path(model_path).suffix

        return extension

    def get_framework(self, extension: Extension):
        if extension == Extension.ONNX:
            return Framework.ONNX
        elif extension == Extension.PT:
            return Framework.PYTORCH
        elif extension == Extension.AIMET:
            return Framework.AIMET
        elif extension == Extension.H5:
            return Framework.TENSORFLOW

    def get_target_extension(self, runtime=Runtime.TFLITE):
        runtime_extensions = {
            Runtime.TFLITE: ".tflite",
            Runtime.QNN_LIB_AARCH64_ANDROID: ".so",  # Deprecated in qai-hub 0.40.0
            Runtime.QNN_CONTEXT_BINARY: ".bin",
            Runtime.QNN_DLC: ".dlc",  # Added in qai-hub 0.40.0
            Runtime.ONNX: ".onnx",
            Runtime.PRECOMPILED_QNN_ONNX: ".zip",
        }

        return runtime_extensions.get(runtime)

    def get_display_runtime(self, runtime: Runtime) -> str:
        RUNTIME_DISPLAY_MAP = {
            Runtime.TFLITE: "TensorFlow Lite",
            Runtime.QNN_LIB_AARCH64_ANDROID: "Qualcomm® AI Engine Direct model library targeting AArch64 Android (Deprecated - use QNN_DLC)",
            Runtime.QNN_CONTEXT_BINARY: "Qualcomm® AI Engine Direct context binary targeting the hardware specified in the compile job.",
            Runtime.QNN_DLC: "Qualcomm® AI Engine Direct DLC (Deep Learning Container) - Recommended for QNN deployment",
            Runtime.ONNX: "ONNX",
            Runtime.PRECOMPILED_QNN_ONNX: "ONNX Runtime model with a pre-compiled QNN context binary.",
        }
        return RUNTIME_DISPLAY_MAP.get(runtime, "Unknown runtime")

    def get_framework_by_runtime(self, runtime: Runtime):
        FRAMEWORK_RUNTIME_MAP = {
            Runtime.TFLITE: Framework.TFLITE,
            Runtime.QNN_LIB_AARCH64_ANDROID: Framework.QNN,
            Runtime.QNN_CONTEXT_BINARY: Framework.QNN,
            Runtime.QNN_DLC: Framework.QNN,  # Added in qai-hub 0.40.0
            Runtime.ONNX: Framework.ONNX,
            Runtime.PRECOMPILED_QNN_ONNX: Framework.QNN,
        }
        return FRAMEWORK_RUNTIME_MAP.get(runtime, "Unknown framework")

    def get_framework_by_model_type(self, model_type: SourceModelType):
        FRAMEWORK_MODEL_TYPE_MAP = {
            SourceModelType.TORCHSCRIPT: Framework.PYTORCH,
            SourceModelType.TFLITE: Framework.TFLITE,
            SourceModelType.ONNX: Framework.ONNX,
            SourceModelType.MLMODEL: Framework.COREML,
            SourceModelType.MLMODELC: Framework.COREML,
            SourceModelType.MLPACKAGE: Framework.COREML,
            SourceModelType.TETRART: Framework.TENSORRT,
            SourceModelType.QNN_LIB_AARCH64_ANDROID: Framework.QNN,
            SourceModelType.QNN_LIB_X86_64_LINUX: Framework.QNN,
            SourceModelType.QNN_CONTEXT_BINARY: Framework.QNN,
            SourceModelType.AIMET_ONNX: Framework.AIMET,
        }
        return FRAMEWORK_MODEL_TYPE_MAP.get(model_type, "Unknown framework")

    def get_display_framework(self, framework: Framework):
        RUNTIME_DISPLAY_MAP = {
            Framework.PYTORCH: "PyTorch",
            Framework.ONNX: "ONNX",
            Framework.ONNXRUNTIME: "ONNXRuntime",
            Framework.AIMET: "AIMET",
            Framework.TENSORFLOW: "TensorFlow",
            Framework.TFLITE: "Tensorflow Lite",
            Framework.COREML: "CoreML",
            Framework.TENSORRT: "TensorRT",
            Framework.QNN: "QNN",
        }
        return RUNTIME_DISPLAY_MAP.get(framework, "Unknown runtime")

    def download_model(self, job: Union[CompileJob, QuantizeJob], filename: str) -> str:
        """
        Download a model from the QAI Hub and handle ONNX zip conversion.

        Args:
            job: The job to download the model from (CompileJob or QuantizeJob).
            filename: The filename to save the model to.

        Returns:
            str: The actual path of the downloaded and processed model.

        Note:
            Since QAI Hub July 2025 update, ONNX models are always downloaded as .zip files
            with external weights. Other formats (.tflite, .so, .bin) are downloaded directly.
            Reference: https://app.aihub.qualcomm.com/docs/hub/release_notes.html#released-july-14-2025

            This function automatically extracts and converts ONNX zip files to single .onnx files
            with embedded weights. Non-ONNX models are returned as-is.
        """
        # Download the model
        downloaded_filename = job.download_target_model(filename=filename)

        # Use the original filename as fallback if download doesn't return a path
        if downloaded_filename is None:
            downloaded_filename = filename

        # Verify file exists
        if not os.path.exists(downloaded_filename):
            raise FileNotFoundError(f"Downloaded file not found: {downloaded_filename}")

        # Only ONNX models are downloaded as .zip (since QAI Hub July 2025 update)
        # Other formats (.tflite, .so, .bin) are downloaded directly
        is_onnx_zip = downloaded_filename.endswith('.onnx.zip') or (
            zipfile.is_zipfile(downloaded_filename) and '.onnx' in filename.lower()
        )

        if is_onnx_zip:
            return self._extract_and_convert_onnx_zip(downloaded_filename)

        # Non-ONNX models: return as-is
        return downloaded_filename

    def _extract_and_convert_onnx_zip(self, zip_path: str) -> str:
        """
        Extract and convert ONNX model from zip file to single .onnx file with embedded weights.

        Args:
            zip_path: Path to the .onnx.zip file.

        Returns:
            str: Path to the converted .onnx file.

        Note:
            Since QAI Hub July 2025 update, ONNX models are always produced with external
            weights in .zip format. This method extracts the zip and saves as a single .onnx file.
            Reference: https://app.aihub.qualcomm.com/docs/hub/release_notes.html#released-july-14-2025
        """
        logger.info(f"Extracting ONNX model from zip: {zip_path}")

        extract_dir = None
        try:
            # Create temporary extraction directory
            extract_dir = f"{zip_path}_extracted"
            os.makedirs(extract_dir, exist_ok=True)

            # Extract the zip file
            with zipfile.ZipFile(zip_path, 'r') as zip_ref:
                zip_ref.extractall(extract_dir)

            # Find ONNX file (recursively search subdirectories)
            onnx_files = []
            for root, _, files in os.walk(extract_dir):
                for file in files:
                    if file.endswith('.onnx'):
                        onnx_files.append(os.path.join(root, file))

            if not onnx_files:
                raise FileNotFoundError(f"No ONNX file found in zip: {zip_path}")

            if len(onnx_files) > 1:
                logger.warning(f"Multiple ONNX files found in zip, using first: {onnx_files[0]}")

            # Load ONNX model
            extracted_onnx_path = onnx_files[0]
            onnx_model = onnx.load(extracted_onnx_path)

            # Determine final path: remove .zip extension
            if zip_path.endswith('.onnx.zip'):
                final_path = zip_path[:-4]  # Remove .zip, keep .onnx
            elif zip_path.endswith('.zip'):
                final_path = zip_path[:-4] + '.onnx'
            else:
                final_path = f"{zip_path}.onnx"

            # Save as single ONNX file with embedded weights
            onnx.save(onnx_model, final_path)
            logger.info(f"ONNX model saved to: {final_path}")

            return final_path

        finally:
            # Always cleanup temporary files
            try:
                if os.path.exists(zip_path):
                    os.remove(zip_path)
                if extract_dir and os.path.exists(extract_dir):
                    shutil.rmtree(extract_dir)
            except Exception as e:
                logger.warning(f"Failed to cleanup temporary files: {e}")



from netspresso.exceptions.common import AdditionalData, PyNPException


class LoadONNXModelException(PyNPException):
    def __init__(self, error_log: str):
        message = "This onnx model is not supported.\nPlease try again with a PyTorch or TensorFlow model."
        super().__init__(
            data=AdditionalData(origin="pynp", error_log=error_log),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )


class UpdateOnnxException(PyNPException):
    def __init__(self, error_log: str):
        message = "Failed to update onnx model.\n\nPlease try again with a PyTorch or TensorFlow model."
        super().__init__(
            data=AdditionalData(origin="pynp", error_log=error_log),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )



from netspresso.enums.base import StrEnum


class GraphOptimizePatternHandler(StrEnum):
    PatternHandlerFoldingGeLU = "PatternHandlerFoldingGeLU",
    PatternHandlerFoldingSilu = "PatternHandlerFoldingSilu",
    PatternHandlerFoldingRMSNorm = "PatternHandlerFoldingRMSNorm",
    PatternHandlerFoldingLayerNorm = "PatternHandlerFoldingLayerNorm",
    PatternHandlerChangeAxisOfSoftmax = "PatternHandlerChangeAxisOfSoftmax",
    PatternHandlerReplaceNegToConv = "PatternHandlerReplaceNegToConv",
    PatternHandlerFuseBNToConv = "PatternHandlerFuseBNToConv",
    PatternHandlerFuseContinuousConcat = "PatternHandlerFuseContinuousConcat",
    PatternHandlerFuseMathIntoConv = "PatternHandlerFuseMathIntoConv",
    PatternHandlerReplaceMatmulToConv = "PatternHandlerReplaceMatmulToConv",
    PatternHandlerFuseMultiReshapeTranspose = "PatternHandlerFuseMultiReshapeTranspose",
    PatternHandlerFuseMultiBranchReshapeTranspose = "PatternHandlerFuseMultiBranchReshapeTranspose",
    PatternHandlerRemoveUselessSlice = "PatternHandlerRemoveUselessSlice"

    @staticmethod
    def get_all():
        return [
            graph_optimize_pattern_handler.value
            for graph_optimize_pattern_handler in GraphOptimizePatternHandler
        ]
